<?php

/**
 *
 * This file is part of hijri calendar exstension
 *
 * @copyright (c) Saeed Hubaishan <http://www.salafitech.net>
 * @license GNU General Public License, version 2 (GPL-2.0)
 *
 * For full copyright and license information, please see
 * the docs/CREDITS.txt file.
 *
 */
namespace hubaishan\hijri;

use hubaishan\hijri\calendar;

/**
 * custom extensions to the phpBB DateTime class to view the hijri calendar
 */
class datetime extends \phpbb\datetime
{
	/**  @var \hubaishan\hijri\calendar  */
	protected static $hcal;
	protected $lang;

	public function __construct($user, $time = 'now', \DateTimeZone $timezone = null)
	{
		self::init_calendar();
		$this->lang = strtok($user->lang_name,'-_');
		parent::__construct($user, $time, $timezone);
	}

	/**
	 * Formats the current date time into the specified format
	 *
	 * @param string $format
	 *        	Optional format to use for output, defaults to users chosen format
	 * @param boolean $force_absolute
	 *        	Force output of a non relative date
	 * @return string Formatted date time
	 */
	public function format($format = '', $force_absolute = false)
	{
		$format = $format ? $format : $this->user->date_format;
		$format = self::format_cache($format, $this->user);
		$relative = ($format['is_short'] && !$force_absolute);
		$now = new self($this->user, 'now', $this->user->timezone);
		
		$timestamp = $this->getTimestamp();
		$now_ts = $now->getTimeStamp();
		
		$delta = $now_ts - $timestamp;
		
		if ($relative)
		{
			/*
			 * Check the delta is less than or equal to 1 hour
			 * and the delta not more than a minute in the past
			 * and the delta is either greater than -5 seconds or timestamp
			 * and current time are of the same minute (they must be in the same hour already)
			 * finally check that relative dates are supported by the language pack
			 */
			if ($delta <= 3600 && $delta > -60 && ($delta >= -5 || (($now_ts / 60) % 60) == (($timestamp / 60) % 60)) && isset($this->user->lang['datetime']['AGO']))
			{
				return $this->user->lang(array(
					'datetime', 
					'AGO'
				), max(0, (int) floor($delta / 60)));
			}
			else
			{
				$midnight = clone $now;
				$midnight->setTime(0, 0, 0);
				
				$midnight = $midnight->getTimestamp();
				
				if ($timestamp <= $midnight + 2 * 86400)
				{
					$day = false;
					
					if ($timestamp > $midnight + 86400)
					{
						$day = 'TOMORROW';
					}
					else if ($timestamp > $midnight)
					{
						$day = 'TODAY';
					}
					else if ($timestamp > $midnight - 86400)
					{
						$day = 'YESTERDAY';
					}
					
					if ($day !== false)
					{
						// Format using the short formatting and finally swap out the relative token placeholder with the correct value
						return str_replace(self::RELATIVE_WRAPPER . self::RELATIVE_WRAPPER, $this->user->lang['datetime'][$day], strtr(self::hformat($format['format_short']), $format['lang']));
					}
				}
			}
		}
		
		return strtr(self::hformat($format['format_long']), $format['lang']);
	}

	/**
	 * Create DateTime object from hijri date
	 *
	 * @param \phpbb\user $user
	 * 			phpbb user
	 * @param integer $year
	 *        	the hijri year
	 * @param integer $month
	 *        	the hijri month
	 * @param integer $day
	 *        	the hijri day
	 * @param \DateTimeZone $timezone
	 *        	Optional the time zone object
	 * @return self datetime object from the given hijri date
	 */
	public static Function createFromHijri($year, $month, $day, $user, \DateTimeZone $timezone = null)
	{
		self::init_calendar();
		$gr_date = self::$hcal->HijriToGregorian($year, $month, $day);
		$d = new static($user, sprintf('%04s', $gr_date['y']) . '-' . $gr_date['m'] . '-' . $gr_date['d'], $timezone);
		return $d;
	}
	/**
	 * Resets the current date of the DateTime object to a different hijri date
	 *
	 * @param integer $year
	 *        	the hijri year
	 * @param integer $month
	 *        	the hijri month
	 * @param integer $day
	 *        	the hijri day
	 * @return void
	 */
	public function setDateHijri($year, $month, $day)
	{
		$gr_date = self::$hcal->HijriToGregorian($year, $month, $day);
		$this->setDate($gr_date['y'], $gr_date['m'], $gr_date['d']);
	}
	
	/**
	 *
	 * @internal
	 *
	 */
	private static function init_calendar()
	{
		global $config, $phpbb_container;
		if (empty(self::$hcal))
		{
			$config_text = $phpbb_container->get('config_text');
			self::$hcal = new calendar(array(
				'umalqura' => ($config['hijri_umalqura'] == '1'), 
				'adj_data' => json_decode($config_text->get('hijri_adj_data'),true)
			));
		}
	}
	

	/**
	 * Formats the current date time into the specified format with hijri calendar support
	 *
	 *
	 *
	 * @param string $format
	 *        	Optional format to use for output, defaults to users chosen format
	 *        	if "_" prefix added to before the character Hijri Calendar will returned
	 *        	
	 * @return string Formatted date time with Hijri Calendar support
	 */
	protected function hformat($format = '')
	{
		if (strpos($format, "_") !== FALSE)
		{
			$hsmonths = array( 1 => 'Muh', 'Saf', 'Rb1', 'Rb2', 'Jm1', 'Jm2', 'Raj', 'Sha', 'Ram', 'Shw', 'Qid', 'Hij');
			list($gy, $gm, $gd) = explode('/', parent::format('Y/m/d'));
			$j = gregoriantojd($gm, $gd, $gy);
			$hy = $hm = $hd = $z = 0;
			self::$hcal->jd2hijri($j, $hy, $hm, $hd, $z);
			// begin of formating
			$str = '';
			$c = str_split($format);
			for ($i = 0, $count_c = sizeof($c); $i < $count_c; $i++)
			{
				
				if ($c[$i] == '\\')
				{
					if ($i < ($count_c - 1))
					{
						$i++;
						$str .= '\\' . $c[$i];
					}
				}
				elseif ($c[$i] == "_")
				{
					$i++;
					
					if ($i >= $count_c)
					{
						break;
					}
					
					switch ($c[$i])
					{
						case 'j' :
							$str .= $hd;
							if (($i + 1) < $count_c && $c[$i + 1] == 'S') {
								if ($this->lang == 'en') {
									$str .= addcslashes(Calendar::english_suffix($hd), 'A..z');
								}
								$i++;
							}
						break;
						
						case 'd' :
							$str .= str_pad($hd, 2, '0', STR_PAD_LEFT);
							if (($i + 1) < $count_c && $c[$i + 1] == 'S') {
								if ($this->lang == 'en') {
									$str .= addcslashes(Calendar::english_suffix($hd), 'A..z');
								}
								$i++;
							}
							break;
						
						case 'z' :
							$str .= $z - 1;
						break;
						
						case 'F' :
							$str .= addcslashes($this->user->lang['HIJRI_MONTHS'][$hm], 'A..z');
						break;
						
						case 'M' :
							if (in_array($this->lang, array('en', 'fr', 'de', 'es', 'pt', 'it', 'en')))
							{
								$str .= addcslashes($hsmonths[$hm], 'A..z');
							}
							else
							{
								$str .= addcslashes($this->user->lang['HIJRI_MONTHS'][$hm], 'A..z');
							}
						break;
						
						case 't' :
							$str .= self::$hcal->days_in_month($hm, $hy);
						break;
						
						case 'm' :
							$str .= str_pad($hm, 2, '0', STR_PAD_LEFT);
						break;
						
						case 'n' :
							$str .= $hm;
						break;
						
						case 'y' :
							$str .= substr($hy, 2);
						break;
						
						case 'Y' :
							$str .= $hy;
						break;
						
						case 'L' :
							$str .= self::$hcal->leap_year($hy);
						break;
						
						case 'S':
							if ($this->lang == 'en') {
								$str .= addcslashes(Calendar::english_suffix($hd), 'A..z');
							}
							break;
						case 'W' :
						case 'o' :
						break;
						
						default :
							$str .= $c[$i];
					}
				}
				else
				{
					$str .= $c[$i];
				}
			}
			$ret_str = \Datetime::format($str);
		}
		else
		{
			$ret_str = \Datetime::format($format);
		}
		
		if (in_array($this->lang, array('ar', 'fa', 'ur', 'ckb')))
		{
			$ret_str = strtr($ret_str, array('AH' => 'ﻫ', 'AD' => 'م'));
		}
		return $ret_str;
	}

	/**
	 * Returns hijri/calendar object witch can be used by other extensions for calendar conversion
	 *
	 *
	 * @return hijri/calendar hijri calendar object
	 */
	public function hijri_calendar()
	{
		return self::$hcal;
	}
}

